from threading import Thread, Lock
from io import StringIO
from subprocess import Popen, PIPE, STDOUT, DEVNULL
import uuid

tasks = {}

class Task(Thread):
	def __init__(self):
		Thread.__init__(self)
		self.task_id = uuid.uuid4()
		self.__mutex = Lock()
		self.__output = StringIO()
		tasks[str(self.task_id)] = self
		
	def print(self, str):
		self.__mutex.acquire()
		self.__output.write(str)
		self.__mutex.release()
		
	def get_output_str(self):
		self.__mutex.acquire()
		result = self.__output.getvalue()
		self.__mutex.release()
		return result
	
	@staticmethod
	def get(task_id):
		return tasks[task_id]
	
class ProcessTask(Task):
	def __init__(self, commands, cwd = None, hide_passwords = []):
		Task.__init__(self)
		self.__commands = commands
		self.__cwd = cwd
		self.__hide_passwords = hide_passwords
		
	def on_fail(self, callback):
		self.__fail_callback = callback
	
	def run(self):
		is_first_command = True
		for command in self.__commands:
			command_display = ' '.join(command)
			for password in self.__hide_passwords:
				command_display = command_display.replace(password, '******')
				
			if not is_first_command:
				self.print('\n')
			is_first_command = False
			self.print('> ' + command_display + '\n')
			
			# We use setsid to make a non-interactive session, otherwise any command that expects an input (confirmation, password, etc.) would hang forever
			self.__process = Popen(['setsid'] + command, cwd = self.__cwd, stdin=DEVNULL, stdout = PIPE, stderr = STDOUT, shell = False)

			for line in self.__process.stdout:
				self.print(line.decode())

			self.__process.wait()

			if self.__process.returncode != 0:
				if self.__fail_callback:
					self.__fail_callback()
				raise Exception("Command failed ("+str(self.__process.returncode)+")\n"+self.get_output_str())